/*
   This version uses color to shade triangles.
   This version also tests converZB2FB().
*/
import java.awt.Color;

public class MainShapesColor
{
   public static void main(String[] args)
   {
      int k = 0; // number of triangles in the triangle fans at the top and bottom
      int n = 0; // number of triangle strips between the top and bottom

      // Check for two command line arguments.
      if (args.length >=2)
      {
         k = Integer.parseInt( args[0] );
         n = Integer.parseInt( args[1] );
      }
      else
      {
         System.err.printf("Usage: %s <k> <n>", MainShapesColor.class.getName());
         System.exit(-1);;
      }

      // Create several Model objects.
      Model m1 = new TriangleModel(3, 3, 3);
      // give each vertex a different color
      m1.triangleList.getFirst().v[0].setColor( Color.red );
      m1.triangleList.getFirst().v[1].setColor( Color.green );
      m1.triangleList.getFirst().v[2].setColor( Color.blue );
      m1.doLighting = false;
      m1.doBackFaceCulling = false; // try setting this true
      m1.doTransparancy = false;
      m1.modelTranslate(-3, -3, -3);

      Model m2 = new CubeModel(1, 1, 1);
      for (Triangle t : m2.triangleList)
         t.setColor( Color.cyan );
      m2.doLighting = false;
      m2.doBackFaceCulling = true;
      m2.doTransparancy = false;
      m2.modelTranslate(-4, 0, 0);

      Model m3 = new SphereModel(k, n, 1);
      // Give the vertices colors based on their coordinates.
      for (Triangle t : m3.triangleList)
      {
         t.v[0].setColor( Math.abs(t.v[0].x), Math.abs(t.v[0].y), Math.abs(t.v[0].z) );
         t.v[1].setColor( Math.abs(t.v[1].x), Math.abs(t.v[1].y), Math.abs(t.v[1].z) );
         t.v[2].setColor( Math.abs(t.v[2].x), Math.abs(t.v[2].y), Math.abs(t.v[2].z) );
      }
      m3.doLighting = false;
      m3.doBackFaceCulling = true;
      m3.doTransparancy = false;
      m3.modelTranslate(-2, 3, 0);

      Model m4 = new ConeModel(k, n, 1, 4);
      for (Triangle t : m4.triangleList)
         t.setColor( Color.blue );
      m4.doLighting = false;
      m4.doBackFaceCulling = true;
      m4.doTransparancy = false;
      m4.modelTranslate(0, -2, 0);

      Model m5 = new CylinderModel(k, n, 1, 4);
      for (Triangle t : m5.triangleList)
         t.setColor( Color.yellow );
      m5.doLighting = false;
      m5.doBackFaceCulling = true;
      m5.doTransparancy = false;
      m5.modelTranslate(3, 0, 0);

      Model m6 = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      m6.doLighting = false;
      m6.doBackFaceCulling = false;
      m6.modelTranslate(0, -5, 0);


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(m3);
      scene.modelList.add(m4);
      scene.modelList.add(m5);
      scene.modelList.add(m6);

      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=0,    eyez=10;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      //System.err.println( scene.toString(3) );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      (new SceneRender(scene, fb)).renderScene();
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_ShapesColor_Frame%02d.ppm", 0) );
      // Look at the contents of the z-buffer.
      FrameBuffer zb = fb.convertZB2FB();
      zb.dumpFB2File( String.format("PPM_ShapesColorZB_Frame%02d.ppm", 0) );

      // Spin the scene 360 degrees.
      for (int i = 1; i <= 36; i++)
      {
         // Rotate the scene by 10 degrees.
         scene.viewRotate(10, 0, 1, 0);
         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         (new SceneRender(scene, fb)).renderScene();
         fb.dumpFB2File(String.format("PPM_ShapesColor_Frame%02d.ppm", i));
         zb = fb.convertZB2FB();
         zb.dumpFB2File( String.format("PPM_ShapesColorZB_Frame%02d.ppm", i) );
      }
   }
}