/**
   Duplicate each Model
*/

public class P0_DuplicateModel implements PipelineStage
{
   public P0_DuplicateModel()
   {
   }

   /**
      Make a copy of the Model so that we can mutate the
      copy as we render it. That is, walk the Model's
      list of Triangle objects and make a copy of each
      Triangle so that future pipeline stages can mutate the
      vertex data in the Triangle.
   */
   public Model processModel(Model model)
   {
      Model newModel = new Model(model); // copy everything except the geometry

      // Copy all the triangles in the model's triangle list.
      for (Triangle t : model.triangleList)
      {
         Triangle tCopy = new Triangle(t);
         newModel.addTriangle( tCopy );
      }
      return newModel;
   }
}