/**
   Shade each fragment created by the rasterizer.
*/

public class P10_FragmentShaderColor implements PipelineStage
{
   public P10_FragmentShaderColor()
   {
   }


   /**
      Walk the Model's list of Triangle objects and for each triangle,
      walk its list of Fragment objects. For each fragment, use
      the barycentric coordinates in the fragment to interpolate
      color data from the triangle's three vertices to the fragment.
   */
   public Model processModel(Model model)
   {
      int triangleCounter = 0;  // useful for debugging
      for (Triangle triangle : model.triangleList)
      {
         triangleCounter++;
         if (null == triangle.tex)  // if this triangle does not have a texture
         {
            // For this triangle, get the color information
            // from each of the three vertices.
            double r0 = triangle.v[0].r;
            double g0 = triangle.v[0].g;
            double b0 = triangle.v[0].b;
            double a0 = triangle.v[0].a;
            double r1 = triangle.v[1].r;
            double g1 = triangle.v[1].g;
            double b1 = triangle.v[1].b;
            double a1 = triangle.v[1].a;
            double r2 = triangle.v[2].r;
            double g2 = triangle.v[2].g;
            double b2 = triangle.v[2].b;
            double a2 = triangle.v[2].a;

            // Walk this triangle's list of fragments.
            for (Fragment frag : triangle.fragments)
            {
               // Get this fragment's barycentric coordinates.
               double alpha = frag.alpha;
               double beta  = frag.beta;
               double gamma = frag.gamma;
               //System.err.printf("P10 alpha = %f, beta = %f, gamma = %f\n", alpha, beta, gamma);

               // Interpolate color data from the triangle's vertices to this fragment.
               frag.r = alpha*r0 + beta*r1 + gamma*r2;
               frag.g = alpha*g0 + beta*g1 + gamma*g2;
               frag.b = alpha*b0 + beta*b1 + gamma*b2;
               frag.a = alpha*a0 + beta*a1 + gamma*a2;
               //System.err.printf("P10 r = %f, g = %f, b = %f\n", frag.r, frag.g, frag.b);
            }
         }
      }
      return model;
   }
}