/**
   Convert vertices from model coordinates to world coordinates.
*/

public class P1_Model2World implements PipelineStage
{
   public P1_Model2World()
   {
   }


   /**
      Walk the Model's list of Triangle objects.
      For each Vertex in each Triangle object,
      use the Model's model matrix to transform
      the Vertex's coordinates (and its normal
      vector) from model to world coordinates.
   */
   public Model processModel(Model model)
   {
      Matrix matrix = model.modelMatrix;
      Matrix invTranspose = matrix.invTranspose();

      // Transform all the triangles in the model's triangle list.
      for (Triangle t : model.triangleList)
      {
         // Transform the vertices.
         t.v[0] = matrix.times(t.v[0]);
         t.v[1] = matrix.times(t.v[1]);
         t.v[2] = matrix.times(t.v[2]);

         // Transform each vertex's normal vector (if there is one).
         if (null != t.v[0].n) t.v[0].n = invTranspose.times(t.v[0].n);
         if (null != t.v[1].n) t.v[1].n = invTranspose.times(t.v[1].n);
         if (null != t.v[2].n) t.v[2].n = invTranspose.times(t.v[2].n);
      }
      return model;
   }
}