/**
   Convert vertices from world coordinates to view (camera) coordinates.
*/

public class P2_World2View implements PipelineStage
{
   Scene scene;

   public P2_World2View(Scene scene)
   {
      this.scene = scene;
   }


   /**
      Walk the Model's list of Triangle objects.
      For each Vertex in each Triangle object,
      use the Scene's view matrix to transform
      the Vertex's coordinates (and its normal
      vector) from world to view coordinates.
    */
   public Model processModel(Model model)
   {
      Matrix viewMatrix = scene.viewMatrix;
      Matrix invTranspose = viewMatrix.invTranspose();

      for (Triangle t : model.triangleList)
      {
         // Transform the vertices.
         t.v[0] = viewMatrix.times(t.v[0]);
         t.v[1] = viewMatrix.times(t.v[1]);
         t.v[2] = viewMatrix.times(t.v[2]);

         // Transform each vertex's normal vector (if there is one).
         if (null != t.v[0].n) t.v[0].n = invTranspose.times(t.v[0].n);
         if (null != t.v[1].n) t.v[1].n = invTranspose.times(t.v[1].n);
         if (null != t.v[2].n) t.v[2].n = invTranspose.times(t.v[2].n);
      }
      return model;
   }
}