/**
   This interface must be implemented by each stage
   of the rendering pipeline.

   The SceneRender class contains a Vector of PipelineStage
   objects and those objects make up a renderer.

   Notice that a stage of the rendering pipeline can
   be implemented by any client program, and then that
   (client defined) stage can be inserted into the
   Vector of PipelineStage objects maintained by the
   SceneRender object. In this way, we have a client
   configurable (and programmable) pipeline.

   Different stages of the pipeline may need different
   extra pieces of information to do their processing.
   Each pipeline stage should use its constructor as a
   way to initialize any extra state information needed
   by that stage. For example in the (default) fixed
   function pipeline, the only extra bits of information
   needed by some of the stages are either a reference to
   a Scene object, or a reference to a FrameBuffer object.
*/
public interface PipelineStage
{
   /**
      Walk the list of Triangle objects in this Model.
      Process each Triangle in an appropriate way.

      This could mean applying a transformation to each
      Vertex object in each Triangle.

      Or (as in the lighting stage) it could mean doing a
      calculation at each Vertex of each Triangle.

      After the rasterization stage, this stage could process
      each Fragment object associated to each of this Model's
      Triangle objects.
    */
   public Model processModel(Model model);
}