/**
   k number of triangles in each triangle fan at the top and base of the cone
   n number of triangle strips between the top and bottom triangle fans
   r radius of the base of the cone
   h height of the cone
*/

public class ConeModel extends Model
{
   public ConeModel(int k, int n, double radius, double h)
   {
      super();

      // Create the cone's geometry.

      // An array of vertices to be used to create triangles.
      Vertex[][] v = new Vertex[n+1][k];

      // Create all the vertices along the cone wall,
      // along with their normal vectors.
      for (int j = 0; j < k; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/k);
         double s = Math.sin(j*(2.0*Math.PI)/k);
         for (int i = 0; i <= n; i++)
         {
            double slantRadius = ((double)(i+1)/(double)(n+1)) * radius;
            v[i][j] = new Vertex( slantRadius*c, h-(i+1)*h/(n+1), slantRadius*s );
            // give the vertex a normal vector
            double l = Math.sqrt( radius*radius + h*h );
            (v[i][j]).n = new Vector(c*h/l, radius/l, s*h/l);
         }
      }
      // Vertex at the center of the top.
      Vertex topCenter = new Vertex(0.0, h, 0.0);
      // Vertex at the center of the bottom.
      Vertex bottomCenter = new Vertex(0.0, 0.0, 0.0);
      bottomCenter.n = new Vector(0.0, -1.0, 0.0);


      // Create all the triangle strips along the cone wall.
      Triangle t;
      for (int i = 0; i < n; i++)
      {
         for (int j = 0; j < k-1; j++)
         {
            t = new Triangle( v[i][j], v[i][j+1], v[i+1][j+1] );
            triangleList.add( t );
            t = new Triangle( v[i+1][j+1], v[i+1][j], v[i][j] );
            triangleList.add( t );
         }
         t = new Triangle( v[i][k-1], v[i][0], v[i+1][0] );
         triangleList.add( t );
         t = new Triangle( v[i+1][0], v[i+1][k-1], v[i][k-1] );
         triangleList.add( t );
      }


      // Create the triangle fan at the top.
      for (int j = 0; j < k-1; j++)
      {
         topCenter.n = v[0][j].n;
         t = new Triangle( topCenter, v[0][j+1], v[0][j] );
         triangleList.add( t );
      }
      topCenter.n = v[0][k-1].n;
      t = new Triangle( topCenter, v[0][0], v[0][k-1] );
      triangleList.add( t );


      // Create the triangle fan at the bottom.
      // (And we need to change the normal vectors.)
      for (int j = 0; j < k-1; j++)
      {
         v[n][j].n =   new Vector(0.0, -1.0, 0.0);
         v[n][j+1].n = new Vector(0.0, -1.0, 0.0);
         t = new Triangle( bottomCenter, v[n][j], v[n][j+1] );
         triangleList.add( t );
      }
      v[n][k-1].n = new Vector(0.0, -1.0, 0.0);
      v[n][0].n   = new Vector(0.0, -1.0, 0.0);
      t = new Triangle( bottomCenter, v[n][k-1], v[n][0] );
      triangleList.add( t );
   }
}//ConeModel