/**
   This is a regular octahedron with side length 1.
   Its center is at the orgin and the center plane is the xz-plane.
   The 5th and 6th vertices are on the y-axis.

   http://en.wikipedia.org/wiki/Regular_octahedron
*/

public class OctahedronModel extends Model
{
   public OctahedronModel()
   {
      super();

      // Create the octahedron's geometry.
      double sqrt3 = Math.sqrt(3.0);
      double sqrt2 = Math.sqrt(2.0);
      Vertex[] v = new Vertex[6];
      v[0] = new Vertex( 0.5, 0,  0.5);  // four vertices around the center plane
      v[1] = new Vertex(-0.5, 0,  0.5);
      v[2] = new Vertex(-0.5, 0, -0.5);
      v[3] = new Vertex( 0.5, 0, -0.5);
      v[4] = new Vertex( 0,  1/sqrt2, 0);   // vertex at the top
      v[5] = new Vertex( 0, -1/sqrt2, 0);   // vertex at the bottom


      // Create 8 triangles.
      Triangle[] t = new Triangle[8];
      Vector n; // used for the normal vectors
      int i = 0;
      // face 1
      n = (v[1].minus(v[0])).crossProduct(v[4].minus(v[0]));
      n = n.normalize();
      v[0].n = v[1].n = v[4].n = n;
      t[i++] = new Triangle(v[0], v[1], v[4]);
      // face 2
      n = (v[2].minus(v[1])).crossProduct(v[4].minus(v[1]));
      n = n.normalize();
      v[1].n = v[2].n = v[4].n = n;
      t[i++] = new Triangle(v[1], v[2], v[4]);
      // face 3
      n = (v[3].minus(v[2])).crossProduct(v[4].minus(v[2]));
      n = n.normalize();
      v[2].n = v[3].n = v[4].n = n;
      t[i++] = new Triangle(v[2], v[3], v[4]);
      // face 4
      n = (v[0].minus(v[3])).crossProduct(v[4].minus(v[3]));
      n = n.normalize();
      v[3].n = v[0].n = v[4].n = n;
      t[i++] = new Triangle(v[3], v[0], v[4]);


      // face 5
      n = (v[1].minus(v[0])).crossProduct(v[5].minus(v[0]));
      n = n.normalize();
      v[0].n = v[1].n = v[5].n = n;
      t[i++] = new Triangle(v[1], v[0], v[5]);
      // face 6
      n = (v[2].minus(v[1])).crossProduct(v[5].minus(v[1]));
      n = n.normalize();
      v[1].n = v[2].n = v[5].n = n;
      t[i++] = new Triangle(v[2], v[1], v[5]);
      // face 7
      n = (v[3].minus(v[2])).crossProduct(v[5].minus(v[2]));
      n = n.normalize();
      v[2].n = v[3].n = v[5].n = n;
      t[i++] = new Triangle(v[3], v[2], v[5]);
      // face 8
      n = (v[0].minus(v[3])).crossProduct(v[5].minus(v[3]));
      n = n.normalize();
      v[3].n = v[0].n = v[5].n = n;
      t[i++] = new Triangle(v[0], v[3], v[5]);


      // Add the new triangles to the model.
      for (i = 0; i < t.length; i++)
      {
         triangleList.add( t[i] );
      }
   }
}//OctahedronModel