/**
   A flat checkerboard in the xz-plane.
*/
import java.awt.Color;

public class PanelModel extends Model
{
   /**
      A checkerboard with two colors.
   */
   public PanelModel(int xMin, int xMax, int zMin, int zMax, Color c1, Color c2)
   {
      this(xMin, xMax, zMin, zMax, c1, c2, null, null);
   }


   /**
      A checkerboard with one color and one texture.
   */
   public PanelModel(int xMin, int xMax, int zMin, int zMax, Color c, Texture tex)
   {
      this(xMin, xMax, zMin, zMax, c, null, tex, null);
   }


   /**
      A checkerboard with two textures.
   */
   public PanelModel(int xMin, int xMax, int zMin, int zMax, Texture tex1, Texture tex2)
   {
      this(xMin, xMax, zMin, zMax, null, null, tex1, tex2);
   }


   /**
      A checkerboard with either colors or textures on each square.
   */
   public PanelModel(int xMin, int xMax, int zMin, int zMax, Color c1, Color c2, Texture tex1, Texture tex2)
   {
      super();

      // Create the checkerboard's geometry.
      Vertex[][] v = new Vertex[(xMax-xMin)+1][(zMax-zMin)+1];

      Vector normal = new Vector(0, 1, 0); // the normal vector
      for (int x = xMin; x <= xMax; x++)
         for (int z = zMin; z <= zMax; z++)
         {
            v[x-xMin][z-zMin] = new Vertex(x, 0, z);
            v[x-xMin][z-zMin].n = normal;
            v[x-xMin][z-zMin].setColorRandom();
         }

      // Create the triangles and give each triangle a color or a texture.
      for (int x = 0; x < xMax-zMin; x++)
      {
         for (int z = 0; z < zMax-zMin; z++)
         {
            Triangle t1 = new Triangle( v[x][z],     v[x][z+1], v[x+1][z+1] );
            Triangle t2 = new Triangle( v[x+1][z+1], v[x+1][z], v[x][z] );
            if ( (x+z)%2 == 0 )
            {
               if (null != c1)
               {
                  t1.setColor( c1 );
                  t2.setColor( c1 );
               }
               else if (null != tex1)
               {
                  t1.setTexture( tex1 );
                  t2.setTexture( tex1 );
               }
            }
            else  // use the other color or texture
            {
               if (null != c2)
               {
                  t1.setColor( c2 );
                  t2.setColor( c2 );
               }
               else if (null != tex2)
               {
                  t1.setTexture( tex2 );
                  t2.setTexture( tex2 );
               }
            }
            triangleList.add( t1 );
            triangleList.add( t2 );
         }
      }
   }
}